"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const child_process_1 = require("child_process");
const fs = require("fs-extra");
const output_1 = require("../../../lib/src/io/output");
class Init extends command_1.Command {
    async run() {
        if (fs.existsSync("api.ts")) {
            this.error(`There is already an API here!`);
            this.exit(1);
        }
        output_1.outputFile(".", "api.ts", `import { api, endpoint, request } from "@zenclabs/api";

@api()
class Api {
  @endpoint({
    method: "POST",
    path: "/users"
  })
  createUser(@request req: CreateUserRequest): CreateUserResponse {
    throw "contract";
  }
}

interface CreateUserRequest {
  firstName: string;
  lastName: string;
}

interface CreateUserResponse {
  success: boolean;
}
`, false);
        output_1.outputFile(".", "tsconfig.json", JSON.stringify({
            compilerOptions: {
                target: "esnext",
                module: "esnext",
                moduleResolution: "node",
                experimentalDecorators: true
            }
        }, null, 2), false);
        output_1.outputFile(".", "package.json", JSON.stringify({}, null, 2), false);
        child_process_1.execSync(`yarn add @zenclabs/api`, {
            stdio: "inherit"
        });
    }
}
Init.description = "Generates the boilerplate for an API.";
Init.examples = [
    `$ api init
Generated the following files:
- api.ts
- tsconfig.json
- package.json
`
];
Init.flags = {
    help: command_1.flags.help({ char: "h" })
};
exports.default = Init;
