"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const assert_never_1 = require("../../assert-never");
function rejectVoidOpenApi2SchemaType(type, errorMessage) {
    const schemaType = openApi2TypeSchema(type);
    if (!schemaType) {
        throw new Error(errorMessage);
    }
    return schemaType;
}
exports.rejectVoidOpenApi2SchemaType = rejectVoidOpenApi2SchemaType;
function openApi2TypeSchema(type) {
    switch (type.kind) {
        case "void":
            return null;
        case "null":
            return null;
        case "boolean":
            return {
                type: "boolean"
            };
        case "boolean-constant":
            return {
                type: "boolean",
                enum: [type.value]
            };
        case "string":
            return {
                type: "string"
            };
        case "string-constant":
            return {
                type: "string",
                enum: [type.value]
            };
        case "number":
            return {
                type: "number"
            };
        case "integer-constant":
            return {
                type: "integer",
                enum: [type.value]
            };
        case "object":
            return Object.entries(type.properties).reduce((acc, [name, type]) => {
                if (type.kind === "optional") {
                    type = type.optional;
                }
                else {
                    acc.required.push(name);
                }
                const schemaType = openApi2TypeSchema(type);
                if (schemaType) {
                    acc.properties[name] = schemaType;
                }
                return acc;
            }, {
                type: "object",
                properties: {},
                required: []
            });
        case "array":
            const itemsType = openApi2TypeSchema(type.elements);
            if (!itemsType) {
                throw new Error(`Unsupported void array`);
            }
            return {
                type: "array",
                items: itemsType
            };
        case "optional":
            throw new Error(`Unsupported top-level optional type`);
        case "union":
            // Please have a look at https://github.com/OAI/OpenAPI-Specification/issues/333
            throw new Error(`Unions are not supported in OpenAPI 2`);
        case "type-reference":
            return {
                $ref: `#/definitions/${type.typeName}`
            };
        default:
            throw assert_never_1.default(type);
    }
}
exports.openApi2TypeSchema = openApi2TypeSchema;
