"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const YAML = require("js-yaml");
const assert_never_1 = require("../../assert-never");
const validator_1 = require("../../validator");
const openapi2_schema_1 = require("./openapi2-schema");
const compact = require("lodash/compact");
const defaultTo = require("lodash/defaultTo");
function generateOpenApiV2(api, format) {
    const contract = openApiV2(api);
    switch (format) {
        case "json":
            return JSON.stringify(contract, null, 2);
        case "yaml":
            return YAML.safeDump(contract);
        default:
            throw assert_never_1.default(format);
    }
}
exports.generateOpenApiV2 = generateOpenApiV2;
function openApiV2(api) {
    return {
        swagger: "2.0",
        tags: [
            {
                name: "TODO"
            }
        ],
        info: {
            version: "0.0.0",
            title: "TODO",
            contact: {
                name: "TODO"
            }
        },
        paths: Object.entries(api.endpoints).reduce((acc, [endpointName, endpoint]) => {
            const openApiPath = endpoint.path
                .map(pathComponent => pathComponent.kind === "dynamic"
                ? `{${pathComponent.name}}`
                : pathComponent.content)
                .join("");
            acc[openApiPath] = acc[openApiPath] || {};
            acc[openApiPath][endpoint.method.toLowerCase()] = {
                operationId: endpointName,
                description: "TODO",
                consumes: consumes(api, endpoint),
                tags: ["TODO"],
                parameters: getParameters(api, endpoint),
                responses: {
                    default: response(api, endpoint.genericErrorType),
                    [(endpoint.successStatusCode || 200).toString(10)]: response(api, endpoint.responseType),
                    ...Object.entries(endpoint.specificErrorTypes).reduce((acc, [errorName, specificError]) => {
                        acc[specificError.statusCode.toString(10)] = response(api, specificError.type);
                        return acc;
                    }, {})
                }
            };
            return acc;
        }, {}),
        definitions: Object.entries(api.types).reduce((acc, [typeName, type]) => {
            acc[typeName] = openapi2_schema_1.rejectVoidOpenApi2SchemaType(type, `Unsupported void type ${typeName}`);
            return acc;
        }, {})
    };
}
exports.openApiV2 = openApiV2;
function getParameters(api, endpoint) {
    const parameters = endpoint.path.map((pathComponent) => pathComponent.kind === "dynamic"
        ? {
            in: "path",
            name: pathComponent.name,
            description: "TODO",
            ...openapi2_schema_1.rejectVoidOpenApi2SchemaType(pathComponent.type, `Unsupported void type for path component ${pathComponent.name}`),
            required: true
        }
        : null).concat([requestBody(api, endpoint.requestType)]);
    return compact(parameters);
}
function consumes(api, endpoint) {
    const contentType = validator_1.isVoid(api, endpoint.requestType) ? null : endpoint.requestContentType;
    return compact([contentType]);
}
function requestBody(api, type) {
    return validator_1.isVoid(api, type) ? null :
        {
            in: "body",
            name: "body",
            description: "TODO",
            required: true,
            schema: defaultTo(openapi2_schema_1.openApi2TypeSchema(type), undefined)
        };
}
function response(api, type) {
    const schemaType = openapi2_schema_1.openApi2TypeSchema(type);
    return {
        ...(schemaType
            ? {
                schema: schemaType
            }
            : {}),
        description: ""
    };
}
