"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const assert_never_1 = require("../../assert-never");
const ts_writer_1 = require("./ts-writer");
function generateTypesSource(api) {
    return ts_writer_1.outputTypeScriptSource(Object.entries(api.types).map(([typeName, type]) => ts.createTypeAliasDeclaration(
    /*decorators*/ undefined, 
    /*modifiers*/ [ts.createModifier(ts.SyntaxKind.ExportKeyword)], typeName, 
    /*typeParameters*/ undefined, typeNode(type))));
}
exports.generateTypesSource = generateTypesSource;
function promiseTypeNode(type) {
    return ts.createTypeReferenceNode("Promise", [typeNode(type)]);
}
exports.promiseTypeNode = promiseTypeNode;
function typeNode(type) {
    switch (type.kind) {
        case "void":
            return VOID_TYPE_NODE;
        case "null":
            return NULL_TYPE_NODE;
        case "boolean":
            return BOOLEAN_TYPE_NODE;
        case "boolean-constant":
            return type.value ? TRUE_TYPE_NODE : FALSE_TYPE_NODE;
        case "string":
            return STRING_TYPE_NODE;
        case "string-constant":
            return stringConstantTypeNode(type.value);
        case "number":
            return NUMBER_TYPE_NODE;
        case "integer-constant":
            return integerConstantTypeNode(type.value);
        case "object":
            return objectTypeNode(type);
        case "array":
            return arrayTypeNode(type);
        case "optional":
            return optionalTypeNode(type);
        case "union":
            return unionTypeNode(type);
        case "type-reference":
            return typeReferenceTypeNode(type);
        default:
            throw assert_never_1.default(type);
    }
}
exports.typeNode = typeNode;
const VOID_TYPE_NODE = ts.createKeywordTypeNode(ts.SyntaxKind.VoidKeyword);
const NULL_TYPE_NODE = ts.createKeywordTypeNode(ts.SyntaxKind.NullKeyword);
const BOOLEAN_TYPE_NODE = ts.createKeywordTypeNode(ts.SyntaxKind.BooleanKeyword);
const TRUE_TYPE_NODE = ts.createLiteralTypeNode(ts.createLiteral(true));
const FALSE_TYPE_NODE = ts.createLiteralTypeNode(ts.createLiteral(false));
const STRING_TYPE_NODE = ts.createKeywordTypeNode(ts.SyntaxKind.StringKeyword);
const NUMBER_TYPE_NODE = ts.createKeywordTypeNode(ts.SyntaxKind.NumberKeyword);
function stringConstantTypeNode(value) {
    return ts.createLiteralTypeNode(ts.createLiteral(value));
}
function integerConstantTypeNode(value) {
    return ts.createLiteralTypeNode(ts.createLiteral(value));
}
function objectTypeNode(type) {
    return ts.createTypeLiteralNode(Object.entries(type.properties).map(([propertyName, propertyType]) => {
        if (propertyType.kind === "void") {
            return ts.createPropertySignature(
            /*modifiers*/ undefined, propertyName, ts.createToken(ts.SyntaxKind.QuestionToken), VOID_TYPE_NODE, 
            /*initializer*/ undefined);
        }
        else if (propertyType.kind === "optional") {
            return ts.createPropertySignature(
            /*modifiers*/ undefined, propertyName, ts.createToken(ts.SyntaxKind.QuestionToken), typeNode(propertyType.optional), 
            /*initializer*/ undefined);
        }
        else {
            return ts.createPropertySignature(
            /*modifiers*/ undefined, propertyName, 
            /*questionToken*/ undefined, typeNode(propertyType), 
            /*initializer*/ undefined);
        }
    }));
}
function arrayTypeNode(type) {
    return ts.createArrayTypeNode(typeNode(type.elements));
}
function optionalTypeNode(type) {
    return ts.createUnionTypeNode([typeNode(type.optional), VOID_TYPE_NODE]);
}
function unionTypeNode(type) {
    return ts.createUnionTypeNode(type.types.map(typeNode));
}
function typeReferenceTypeNode(type) {
    return ts.createTypeReferenceNode(type.typeName, /*typeArguments*/ undefined);
}
