import { Type } from "../../models";
import { HttpContentType } from "@zenclabs/api";
export declare function rejectVoidOpenApi3SchemaType(type: Type, errorMessage: string): OpenAPI3SchemaType;
export declare function openApiV3ContentTypeSchema(contentType: HttpContentType, type: Type): OpenAPI3SchemaContentType;
export declare function openApi3TypeSchema(type: Type): OpenAPI3SchemaType | null;
export declare type OpenAPI3SchemaType = OpenAPI3SchemaTypeObject | OpenAPI3SchemaTypeArray | OpenAPI3SchemaTypeOneOf | OpenAPI3SchemaTypeNull | OpenAPI3SchemaTypeString | OpenAPI3SchemaTypeNumber | OpenAPI3SchemaTypeInteger | OpenAPI3SchemaTypeBoolean | OpenAPI3SchemaTypeReference;
export interface OpenAPI3BaseSchemaType {
    nullable?: boolean;
    discriminator?: {
        propertyName: string;
        mapping: {
            [value: string]: OpenAPI3SchemaType;
        };
    };
}
export declare type OpenAPI3SchemaContentType = OpenAPI3SchemaApplicationJsonContentType | OpenAPI3SchemaTextHtmlContentType;
export interface OpenAPI3SchemaApplicationJsonContentType extends OpenAPI3BaseSchemaType {
    content: {
        'application/json': {
            schema: OpenAPI3SchemaType | null;
        };
    };
}
export interface OpenAPI3SchemaTextHtmlContentType extends OpenAPI3BaseSchemaType {
    content: {
        'text/html': {
            schema: OpenAPI3SchemaTypeString;
        };
    };
}
export interface OpenAPI3SchemaTypeObject extends OpenAPI3BaseSchemaType {
    type: "object";
    properties: {
        [name: string]: OpenAPI3SchemaType;
    };
    required?: string[];
}
export interface OpenAPI3SchemaTypeArray extends OpenAPI3BaseSchemaType {
    type: "array";
    items: OpenAPI3SchemaType;
}
export interface OpenAPI3SchemaTypeOneOf extends OpenAPI3BaseSchemaType {
    oneOf: OpenAPI3SchemaType[];
}
export interface OpenAPI3SchemaTypeNull extends OpenAPI3BaseSchemaType {
}
export interface OpenAPI3SchemaTypeString extends OpenAPI3BaseSchemaType {
    type: "string";
    enum?: string[];
}
export interface OpenAPI3SchemaTypeNumber extends OpenAPI3BaseSchemaType {
    type: "number";
    enum?: number[];
}
export interface OpenAPI3SchemaTypeInteger extends OpenAPI3BaseSchemaType {
    type: "integer";
    enum?: number[];
}
export interface OpenAPI3SchemaTypeBoolean extends OpenAPI3BaseSchemaType {
    type: "boolean";
    enum?: boolean[];
}
export interface OpenAPI3SchemaTypeReference extends OpenAPI3BaseSchemaType {
    $ref: string;
}
