"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const YAML = require("js-yaml");
const assert_never_1 = require("../../assert-never");
const validator_1 = require("../../validator");
const openapi3_schema_1 = require("./openapi3-schema");
const identity = require("lodash/identity");
const compact = require("lodash/compact");
const pickBy = require("lodash/pickBy");
const defaultTo = require("lodash/defaultTo");
function generateOpenApiV3(api, format) {
    const contract = openApiV3(api);
    switch (format) {
        case "json":
            return JSON.stringify(contract, null, 2);
        case "yaml":
            return YAML.safeDump(contract);
        default:
            throw assert_never_1.default(format);
    }
}
exports.generateOpenApiV3 = generateOpenApiV3;
function openApiV3(api) {
    return {
        openapi: "3.0.0",
        tags: [
            {
                name: "TODO"
            }
        ],
        info: {
            version: "0.0.0",
            title: "TODO",
            contact: {
                name: "TODO"
            }
        },
        paths: Object.entries(api.endpoints).reduce((acc, [endpointName, endpoint]) => {
            const openApiPath = endpoint.path
                .map(pathComponent => pathComponent.kind === "dynamic"
                ? `{${pathComponent.name}}`
                : pathComponent.content)
                .join("");
            acc[openApiPath] = acc[openApiPath] || {};
            acc[openApiPath][endpoint.method.toLowerCase()] = {
                operationId: endpointName,
                description: "TODO",
                tags: ["TODO"],
                parameters: compact(endpoint.path.map((pathComponent) => pathComponent.kind === "dynamic"
                    ? {
                        in: "path",
                        name: pathComponent.name,
                        description: "TODO",
                        required: true,
                        schema: openapi3_schema_1.rejectVoidOpenApi3SchemaType(pathComponent.type, `Unsupported void type for path component ${pathComponent.name}`)
                    }
                    : null)),
                ...pickBy({
                    requestBody: validator_1.isVoid(api, endpoint.requestType)
                        ? undefined
                        : defaultTo(openapi3_schema_1.openApiV3ContentTypeSchema(defaultTo(endpoint.requestContentType, "application/json"), endpoint.requestType), undefined)
                }, identity),
                responses: {
                    default: response(api, endpoint.genericErrorType),
                    [(endpoint.successStatusCode || 200).toString(10)]: response(api, endpoint.responseType),
                    ...Object.entries(endpoint.specificErrorTypes).reduce((acc, [errorName, specificError]) => {
                        acc[specificError.statusCode.toString(10)] = response(api, specificError.type);
                        return acc;
                    }, {})
                }
            };
            return acc;
        }, {}),
        components: {
            schemas: Object.entries(api.types).reduce((acc, [typeName, type]) => {
                acc[typeName] = openapi3_schema_1.rejectVoidOpenApi3SchemaType(type, `Unsupported void type ${typeName}`);
                return acc;
            }, {})
        }
    };
}
exports.openApiV3 = openApiV3;
function response(api, type) {
    const schemaType = openapi3_schema_1.openApi3TypeSchema(type);
    return {
        ...(schemaType
            ? {
                content: {
                    "application/json": {
                        schema: schemaType
                    }
                }
            }
            : {}),
        description: ""
    };
}
