"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const compact = require("lodash/compact");
const uniq = require("lodash/uniq");
function gatherTypes(endpoint) {
    return uniq([
        ...compact(endpoint.path.map(p => (p.kind === "dynamic" ? p.type : null))),
        ...Object.values(endpoint.headers).map(h => h.type),
        endpoint.requestType,
        endpoint.responseType,
        endpoint.genericErrorType,
        ...Object.values(endpoint.specificErrorTypes).map(s => s.type)
    ]);
}
exports.gatherTypes = gatherTypes;
exports.VOID = {
    kind: "void"
};
exports.NULL = {
    kind: "null"
};
exports.BOOLEAN = {
    kind: "boolean"
};
function booleanConstant(value) {
    return {
        kind: "boolean-constant",
        value
    };
}
exports.booleanConstant = booleanConstant;
exports.STRING = {
    kind: "string"
};
function stringConstant(value) {
    return {
        kind: "string-constant",
        value
    };
}
exports.stringConstant = stringConstant;
exports.NUMBER = {
    kind: "number"
};
function integerConstant(value) {
    if (value !== Math.round(value)) {
        throw new Error(`Invalid integer: ${value}`);
    }
    return {
        kind: "integer-constant",
        value
    };
}
exports.integerConstant = integerConstant;
function objectType(properties) {
    return {
        kind: "object",
        properties
    };
}
exports.objectType = objectType;
function arrayType(elements) {
    return {
        kind: "array",
        elements
    };
}
exports.arrayType = arrayType;
function optionalType(type) {
    if (type.kind === "optional") {
        return type;
    }
    return {
        kind: "optional",
        optional: type
    };
}
exports.optionalType = optionalType;
function unionType(...types) {
    types = uniq(types.map(extractPossibleTypes).reduce((acc, curr) => [...acc, ...curr], []));
    if (types.length === 0) {
        return exports.VOID;
    }
    else if (types.length === 1) {
        return types[0];
    }
    else {
        return {
            kind: "union",
            types
        };
    }
}
exports.unionType = unionType;
function extractPossibleTypes(type) {
    if (type.kind === "union") {
        return type.types
            .map(extractPossibleTypes)
            .reduce((acc, curr) => [...acc, ...curr], []);
    }
    return [type];
}
function typeReference(typeName) {
    return {
        kind: "type-reference",
        typeName
    };
}
exports.typeReference = typeReference;
